@@ -1,12 +1,15 @@
 package darumaDump;
 
 import mispClient.MispClient;
+import mispClient.DirectMispClient;
+import mispClient.HttpMispClient;
 import mispClient.MispCommandException;
 import mispClient.commands.GetCapabilitiesMispCommand;
 import mispClient.commands.DescribeFeatureTypeMispCommand;
 import mispClient.commands.GetFeatureDataOnlyMispCommand;
 
 import util.FilePathConverter;
+import util.InputStreamStringBuilder;
 
 import daruma.storage_manager.type_definition.ElementName;
 import daruma.xml.URI;
@@ -15,9 +18,12 @@ import daruma.xml.UniversalName;
 import daruma.util.CharacerCodeConstant;
 import daruma.util.FatalException;
 
+import java.io.InputStream;
 import java.io.OutputStream;
 import java.io.PrintStream;
+import java.io.BufferedInputStream;
 import java.io.File;
+import java.io.FileInputStream;
 import java.io.FileOutputStream;
 import java.io.FileNotFoundException;
 import java.io.PrintWriter;
@@ -35,30 +41,41 @@ public class DarumaDump
 
 	private	static final String	DEFAULT_MISP_HOST = "localhost";
 	private	static final int	DEFAULT_MISP_PORT = 5050;
+	private	static final boolean	DEFAULT_HTTP = false;
+	private	static final String	DEFAULT_HTTP_ADDRESS_LOCAL_PART
+								    = "daruma";
 
 	private	static final String	DEFAULT_SCHEMA_DUMP_FILE = "schema.xml";
 
 	private	String		host;
 	private	int		port;
+	private	boolean		http;
+	private	String		httpAddressLocalPart;
 	private	OutputStream	dataOutputStream;
 	private	OutputStream	schemaOutputStream;
 	private	String		featureDumpDir;
 	private	UniversalName	specifiedFeatureName;
+	private	String		filterFilePath;
 	private boolean		debugFlag;
 
 	public	DarumaDump( String  host ,  int  port ,
+			    boolean  http, String  httpAddressLocalPart,
 			    OutputStream  dataOutputStream ,
 			    OutputStream  schemaOutputStream ,
-			    String featureDumpDir,
-			    UniversalName specifiedFeatureName ,
-			    boolean debugFlag )
+			    String  featureDumpDir,
+			    UniversalName  specifiedFeatureName ,
+			    String  filterFilePath ,
+			    boolean  debugFlag )
 	{
 		this.host = host;
 		this.port = port;
+		this.http = http;
+		this.httpAddressLocalPart = httpAddressLocalPart;
 		this.dataOutputStream = dataOutputStream;
 		this.schemaOutputStream = schemaOutputStream;
 		this.featureDumpDir = featureDumpDir;
 		this.specifiedFeatureName = specifiedFeatureName;
+		this.filterFilePath = filterFilePath;
 		this.debugFlag = debugFlag;
 	}
 
@@ -66,8 +83,34 @@ public class DarumaDump
 	{
 		try
 		{
-			MispClient	client = new MispClient( this.host ,
-								 this.port );
+			MispClient client;
+			if ( this.http )
+			{
+			    client = new HttpMispClient
+					 ( this.host ,
+					   this.port ,
+					   this.httpAddressLocalPart );
+			}
+			else
+			{
+			    client = new DirectMispClient( this.host ,
+							   this.port );
+			}
+
+
+			//
+			// read filter file
+			//
+			String filter = null;
+			{
+			    if ( this.filterFilePath != null )
+			    {
+				filter = new InputStreamStringBuilder
+					 ( new FileInputStream
+					   ( filterFilePath ) ).toString();
+			    }
+			}
+
 
 			//
 			// GetCapabilities
@@ -120,7 +163,8 @@ public class DarumaDump
 			{
 				this.outputAllFeature( client ,
 						       this.dataOutputStream ,
-						       features );
+						       features ,
+						       filter );
 			}
 			else
 			{
@@ -155,7 +199,7 @@ public class DarumaDump
 					( this.featureDumpDir,
 					  FilePathConverter.escapePathString
 					  ( f.toString() + ".misp" ) ) );
-				this.outputFeature( client , file , f );
+				this.outputFeature( client , file , f , filter );
 				file.close();
 				System.err.println( " done");
 			    }
@@ -175,7 +219,7 @@ public class DarumaDump
 		}
 		catch( IOException  e )
 		{
-			e.printStackTrace();
+			System.err.println( "IO error: " + e.getMessage() );
 			System.exit( 1 );
 		}
 	}
@@ -184,7 +228,8 @@ public class DarumaDump
 	private	static	void	outputFeature
 					( MispClient  client ,
 					  OutputStream  dataOutputStream ,
-					  ElementName  feature )
+					  ElementName  feature ,
+					  String  filter )
 						throws MispCommandException ,
 						       ConnectException ,
 						       IOException
@@ -202,6 +247,7 @@ public class DarumaDump
 
 		client.issue( new GetFeatureDataOnlyMispCommand
 			      ( feature ,
+				filter ,
 				dataOutputStream ,
 				CharacerCodeConstant.JAVA_UTF8 ) );
 
@@ -214,7 +260,8 @@ public class DarumaDump
 	private	static	void	outputAllFeature
 					( MispClient  client ,
 					  OutputStream  dataOutputStream ,
-					  List<ElementName>  features )
+					  List<ElementName>  features ,
+					  String  filter )
 						throws MispCommandException ,
 						       ConnectException ,
 						       IOException
@@ -234,6 +281,7 @@ public class DarumaDump
 		{
 			client.issue( new GetFeatureDataOnlyMispCommand
 				      ( f ,
+					filter ,
 					dataOutputStream ,
 					CharacerCodeConstant.JAVA_UTF8 ) );
 		}
@@ -284,6 +332,7 @@ public class DarumaDump
 		opt = new DarumaDumpOptionAnalyzer
 			( args , PROGRAM_NAME ,
 			  DEFAULT_MISP_HOST , DEFAULT_MISP_PORT ,
+			  DEFAULT_HTTP, DEFAULT_HTTP_ADDRESS_LOCAL_PART,
 			  DEFAULT_SCHEMA_DUMP_FILE );
 
 		if ( ! opt.analyze() )
@@ -318,10 +367,13 @@ public class DarumaDump
 
 		darumaDump = new DarumaDump( opt.getHost() ,
 					     opt.getPort() ,
+					     opt.getHttp() ,
+					     opt.getHttpAddressLocalPart() ,
 					     System.out ,
 					     schemaOutputStream,
 					     opt.getFeatureDumpDir(),
 					     opt.getFeatureName(),
+					     opt.getFilterFilePath(),
 					     opt.getDebug() );
 		try
 		{
@@ -354,7 +406,11 @@ public class DarumaDump
 		private	String		programName;
 		private	String		host;
 		private	int		port;
-		private UniversalName	featureName = null;
+		private	boolean		http;
+		private	String		httpAddressLocalPart;
+
+		private	UniversalName	featureName = null;
+		private	String		filterFilePath = null;
 
 		private	boolean	helpFlag  = false;
 		private	boolean	debugFlag = false;
@@ -362,6 +418,8 @@ public class DarumaDump
 		private	String	schemaDumpFile = null;
 		private	String	featureDumpDir = null;
 
+		private	boolean	defaultHttp = false;
+		private	String	defaultHttpAddressLocalPart = null;
 		private	String	defaultSchemaDumpFile = null;
 
 		public	DarumaDumpOptionAnalyzer
@@ -369,16 +427,24 @@ public class DarumaDump
 			  final String  programName ,
 			  final String  defaultMispHost ,
 			  final int  defaultMispPort ,
+			  final boolean defaultHttp,
+			  final String defaultHttpAddressLocalPart,
 			  final String  defaultSchemaDumpFile )
 		{
 			this.args = args.clone();
 			this.programName = programName;
 			this.host = defaultMispHost;
 			this.port = defaultMispPort;
+			this.http = defaultHttp;
+			this.httpAddressLocalPart = defaultHttpAddressLocalPart;
 			this.featureName = null;
+			this.filterFilePath = null;
 			this.schemaDumpFile = defaultSchemaDumpFile;
 			this.featureDumpDir = null;
 
+			this.defaultHttp = defaultHttp;
+			this.defaultHttpAddressLocalPart
+			    = defaultHttpAddressLocalPart;
 			this.defaultSchemaDumpFile = defaultSchemaDumpFile;
 		}
 
@@ -436,6 +502,25 @@ public class DarumaDump
 						return( false );
 					}
 				}
+				else if ( arg.equals( "--http" )
+				       || arg.equals( "-H" ) )
+				{
+					this.http = true;
+				}
+				else if ( arg.equals
+					      ( "--http-address-local-part" )
+				       || arg.equals( "-L" ) )
+				{
+					if ( numArgs < 1 )
+					{
+						return( false );
+					}
+
+					index ++;
+
+					this.httpAddressLocalPart
+					    = this.args[index];
+				}
 				else if ( arg.equals( "--schema-dump-file" ) )
 				{
 					if ( numArgs < 1 )
@@ -472,6 +557,18 @@ public class DarumaDump
 
 					index += 2;
 				}
+				else if ( arg.equals( "--filter-file" ) )
+				{
+					if ( numArgs < 1 )
+					{
+						return( false );
+					}
+
+					this.filterFilePath
+					    = this.args[index + 1];
+
+					index += 1;
+				}
 				else
 				{
 					System.err.println
@@ -483,21 +580,6 @@ public class DarumaDump
 				index ++;
 			}
 
-			return( this.check() );
-		}
-
-		public	boolean	check()
-		{
-			/*
-			if ( this.schemaDumpFile == null )
-			{
-				System.err.println
-					( "schema file not specified" );
-
-				return( false );
-			}
-			*/
-
 			return( true );
 		}
 
@@ -511,17 +593,21 @@ public class DarumaDump
 			mes += "Possible options are:"    + "\n";
 			mes += "      --help"             + "\n";
 			mes += "      --debug"            + "\n";
-			mes += "   -h,--host HOST"        + "\n";
-			mes += "   -p,--port PORT"        + "\n";
+			mes += "  -h, --host HOST"        + "\n";
+			mes += "  -p, --port PORT"        + "\n";
+			mes += "  -H, --http"
+			    + " (default: " + this.defaultHttp + ")\n";
+			mes += "  -L, --http-address-local-part PATH"
+			    + " (default: " + this.defaultHttpAddressLocalPart
+			    + ")\n";
 			mes += "      --schema-dump-file FILE"
 			    + " (default: " + this.defaultSchemaDumpFile + ")"
 			    + "\n";
 			mes += "      --feature-dump-dir DIR"
-			    + " (default: none, standard output)"
-			    + "\n";
+			    + " (default: none, standard output)\n";
 			mes += "      --feature NAMESPACE LOCAL_NAME"
-			    + " (default: all features)"
-			    + "\n";
+			    + " (default: all features)\n";
+			mes += "      --filter-file FILE\n";
 
 			PrintStream	p = new PrintStream( out );
 			p.print( mes );
@@ -547,11 +633,26 @@ public class DarumaDump
 			return( this.port );
 		}
 
+		public	boolean	getHttp()
+		{
+			return( this.http );
+		}
+
+		public	String	getHttpAddressLocalPart()
+		{
+			return( this.httpAddressLocalPart );
+		}
+
 		public	UniversalName	getFeatureName()
 		{
 			return( this.featureName );
 		}
 
+		public	String	getFilterFilePath()
+		{
+		    return( this.filterFilePath );
+		}
+
 		public	String	getSchemaDumpFile()
 		{
 			return( this.schemaDumpFile );
