@@ -8,6 +8,7 @@ import java.io.InputStream;
 import java.io.OutputStream;
 import java.io.IOException;
 import java.io.ByteArrayInputStream;
+import java.io.PrintStream;
 import java.io.PrintWriter;
 import java.util.Map;
 import java.util.HashMap;
@@ -21,6 +22,7 @@ import daruma.sql.DatabaseConnectionException;
 import daruma.util.LogWriter;
 import daruma.util.PropertyReader;
 import daruma.util.FatalException;
+import daruma.util.PortNumberValidator;
 import daruma.xml.util.XMLFormatConverter;
 import daruma.wfs.SOAPFaultDocumentBuilder;
 
@@ -33,6 +35,11 @@ import daruma.xml.util.DOMDocumentFactory;
 import org.w3c.dom.Document;
 */
 import daruma.auth.AuthenticationInfo;
+import daruma.auth.PermissionManager;
+
+import org.eclipse.jetty.server.Server;
+import org.eclipse.jetty.servlet.ServletContextHandler;
+import org.eclipse.jetty.servlet.ServletHolder;
 
 public class DarumaServlet extends HttpServlet
 {
@@ -40,23 +47,28 @@ public class DarumaServlet extends HttpServlet
 
     private long connectionID = 0;
 
-    private void initializeEnvironment() throws FatalException
+    private static boolean initialized = false;
+    private synchronized void initializeEnvironment() throws FatalException
     {
 	//
-	// initialize global Classes
+	// initialize global classes
 	//
-	daruma.util.PropertyReader.initialize
-	    ( this.getClass().getClassLoader().getResourceAsStream
-	      ( daruma.util.PropertyReader.getDefaultPropertyFile() ) );
-	daruma.util.LogWriter.initialize
-	    ( this.getClass().getClassLoader().getResource
-	      ( daruma.util.PropertyReader.getDefaultPropertyFile() ) );
+	if ( ! initialized )
+	{
+	    daruma.util.PropertyReader.initialize
+		( this.getClass().getClassLoader().getResourceAsStream
+		  ( daruma.util.PropertyReader.getDefaultPropertyFile() ) );
+	    daruma.util.LogWriter.initialize
+		( this.getClass().getClassLoader().getResource
+		  ( daruma.util.PropertyReader.getDefaultPropertyFile() ) );
 
-	ImplementationSwitches.createInstance();
+	    ImplementationSwitches.createInstance();
+
+	    this.initialized = true;
+	}
     }
 
 
-
     @Override
     public void doGet( HttpServletRequest request,
 		       HttpServletResponse response )
@@ -81,11 +93,11 @@ public class DarumaServlet extends HttpServlet
 
 	AuthenticationInfo authInfo = this.getAuthenticatedInfo( request );
 
-	daruma.util.LogWriter.qwrite("INFO",
-				     "auth = ",
-				     authInfo.certificated(), ", ",
-				     "user = [",
-				     authInfo.getAuthenticatedUser(), "]" );
+	LogWriter.qwrite("INFO",
+			 "auth = ",
+			 authInfo.certificated(), ", ",
+			 "user = [",
+			 authInfo.getAuthenticatedAccount().getName(), "]" );
 
 
 	//
@@ -551,4 +563,204 @@ public class DarumaServlet extends HttpServlet
 
 	return value;
     }
+
+
+    //
+    // jetty
+    //
+    private static final int DEFAULT_JETTY_PORT = 8080;
+    private static final String DEFAULT_HTTP_LOCAL_PART = "/daruma";
+
+    public static void main( String[] args )
+    {
+	//
+	// alanyze options
+	//
+	DarumaServletOptionAnalyzer opt
+	    = new DarumaServletOptionAnalyzer( args,
+					       "DarumaServlet",
+					       DEFAULT_JETTY_PORT,
+					       DEFAULT_HTTP_LOCAL_PART );
+
+	if ( ! opt.analyze() )
+	{
+	    opt.printHelp( System.err );
+	    System.exit( 1 );
+	}
+	else if ( opt.getHelp() )
+	{
+	    opt.printHelp( System.err );
+	    System.exit( 0 );
+	}
+	else if ( opt.getVersion() )
+	{
+	    System.out.println( DarumaVersion.getVersion() );
+	    System.exit( 0 );
+	}
+
+
+	//
+	// run servlet
+	//
+	Server server = new Server( opt.getPort() );
+	ServletContextHandler context
+	    = new ServletContextHandler( ServletContextHandler.SESSIONS );
+	server.setHandler( context );
+
+	context.addServlet( new ServletHolder( new DarumaServlet() ),
+			    opt.getHttpLocalPart() );
+
+	try
+	{
+	    server.start();
+	    server.join();
+	}
+	catch( Exception e )
+	{
+	    e.printStackTrace();
+	}
+    }
+
+    public static class DarumaServletOptionAnalyzer
+    {
+	private String[] args;
+	private String programName;
+
+	private boolean helpFlag;
+	private boolean versionFlag;
+
+	private int defaultPortNumber;
+	private int portNumber;
+
+	private String defaultHttpLocalPart;
+	private String httpLocalPart;
+
+	DarumaServletOptionAnalyzer( String[] args, String programName,
+				     int defaultPortNumber,
+				     String defaultHttpLocalPart )
+	{
+	    this.args = args.clone();
+	    this.programName = programName;
+
+	    this.helpFlag = false;
+	    this.versionFlag = false;
+
+	    this.defaultPortNumber = defaultPortNumber;
+	    this.portNumber = defaultPortNumber;
+
+	    this.defaultHttpLocalPart = defaultHttpLocalPart;
+	    this.httpLocalPart = defaultHttpLocalPart;
+	}
+
+	public boolean analyze()
+	{
+	    int index = 0;
+	    while( index < this.args.length )
+	    {
+		final String arg = this.args[index];
+		int numArgs = args.length - index - 1;
+
+		if ( arg.equals( "--help" ) )
+		{
+		    this.helpFlag = true;
+		}
+		else if ( arg.equals( "--version" ) )
+		{
+		    this.versionFlag = true;
+		}
+		else if ( arg.equals( "--port" ) )
+		{
+		    if ( numArgs < 1 )
+		    {
+			return false;
+		    }
+
+		    index ++;
+		    this.portNumber = this.parsePortNumber( args[index] );
+
+		    if ( this.portNumber < 0 )
+		    {
+			return false;
+		    }
+		}
+		else if ( arg.equals( "--http-local-part" ) )
+		{
+		    if ( numArgs < 1 )
+		    {
+			return false;
+		    }
+
+		    index ++;
+		    this.httpLocalPart = args[index];
+		}
+
+		index ++;
+	    }
+
+	    return true;
+	}
+
+	private int parsePortNumber( String value )
+	{
+	    int p;
+
+	    try
+	    {
+		p = Integer.parseInt( value );
+	    }
+	    catch( NumberFormatException e )
+	    {
+		LogWriter.qwrite( "ERROR", "invalid port number: ", value );
+		System.err.println( "invalid port number: " + value );
+		return -1;
+	    }
+
+	    if ( ! PortNumberValidator.check( p ) )
+	    {
+		LogWriter.qwrite( "ERROR",
+				  "invalid port number, out of range: ",
+				  value );
+		System.err.println( "invalid port number, out of range: "
+				    + value );
+		return -1;
+	    }
+
+	    return p;
+	}
+
+	public void printHelp( OutputStream out )
+	{
+	    String mes = "";
+	    mes += "Usage: " + this.programName + " [options]"         + "\n";
+	    mes += "Possible options are:"                             + "\n";
+	    mes += "      --help"                                      + "\n";
+	    mes += "      --version"                                   + "\n";
+	    mes += "      --port PORT"
+		+ " (default: " + this.defaultPortNumber + ")"         + "\n";
+	    mes += "      --http-local-part"
+		+ " (default: \"" + this.defaultHttpLocalPart + "\")"  + "\n";
+
+	    new PrintStream( out ).print( mes );
+	}
+
+	public boolean getHelp()
+	{
+	    return this.helpFlag;
+	}
+
+	public boolean getVersion()
+	{
+	    return this.versionFlag;
+	}
+
+	public int getPort()
+	{
+	    return this.portNumber;
+	}
+
+	public String getHttpLocalPart()
+	{
+	    return this.httpLocalPart;
+	}
+    }
 }
