@@ -20,6 +20,7 @@ import daruma.storage_manager.type_definition.types.ChoiceTypeDefinition;
 import daruma.storage_manager.type_definition.types.MultipleOccursElementTypeDefinition;
 import daruma.storage_manager.type_definition.types.ComplexTypeDefinition;
 import daruma.storage_manager.type_definition.types.StringTypeDefinition;
+import daruma.storage_manager.type_definition.types.IntegerTypeDefinition;
 
 import daruma.xml.UniversalName;
 import daruma.xml.QName;
@@ -204,13 +205,15 @@ public class XMLSchemaTypeParser
 				    + " is not a schema element" );
 			}
 
-			if ( r.getLocalName().equals( "maxLength" )
-			  || r.getLocalName().equals( "minLength" ) )
+			String localName = r.getLocalName();
+
+			if ( localName.equals( "maxLength" )
+			  || localName.equals( "minLength" ) )
 			{
 				if ( ! r.hasAttribute( "value" ) )
 				{
 					throw new XMLSchemaParserException
-					( r.getLocalName()
+					( localName
 					  + " in namespace "
 					  + r.getNamespaceURI()
 					  + " does not have attribute"
@@ -239,32 +242,87 @@ public class XMLSchemaTypeParser
 					  + " is not a unsigned integer" );
 				}
 
-				// XXX
+				// XXX: should check type can be restricted
 				if ( type instanceof StringTypeDefinition )
 				{
 					result = ((StringTypeDefinition)result)
 								.deepCopy();
 
-					if ( r.getLocalName().equals
+					if ( localName.equals
 					     ( "minLength" ) )
 					{
 						((StringTypeDefinition)result)
 							.setMinLength( limit );
 					}
-					else if ( r.getLocalName().equals
+					else if ( localName.equals
 						  ( "maxLength" ) )
 					{
 						((StringTypeDefinition)result)
 							.setMaxLength( limit );
 					}
 				}
+				else
+				{
+					throw new XMLSchemaParserException
+					( "type " + type
+					  + " cannot be restricted by " + r );
+				}
 			}
-			else if ( r.getLocalName().equals( "enumeration" ) )
+			else if ( localName.equals( "minExclusive" )
+			       || localName.equals( "maxExclusive" )
+			       || localName.equals( "minInclusive" )
+			       || localName.equals( "maxInclusive" ) )
+			{
+			    String value = r.getAttribute( "value" );
+
+			    if ( type instanceof IntegerTypeDefinition )
+			    {
+				long threshold;
+
+				try
+				{
+				    threshold = Long.decode( value );
+				}
+				catch( NumberFormatException  e )
+				{
+				    throw new XMLSchemaParserException
+					( "\"" + value + "\""
+					  + " is not a valid integer" );
+				}
+
+				IntegerTypeDefinition
+				    t = ((IntegerTypeDefinition)result)
+							    .deepCopy();
+				result = t;
+
+				if ( localName.equals( "minExclusive" ) )
+				{
+				    t.setMinInclusiveValue( threshold + 1 );
+				}
+				else if ( localName.equals( "maxExclusive" ) )
+				{
+				    t.setMaxInclusiveValue( threshold - 1 );
+				}
+				else if ( localName.equals( "minInclusive" ) )
+				{
+				    t.setMinInclusiveValue( threshold );
+				}
+				else if ( localName.equals( "maxInclusive" ) )
+				{
+				    t.setMaxInclusiveValue( threshold );
+				}
+			    }
+			    else
+			    {
+				// XXX: not implemented yet
+			    }
+			}
+			else if ( localName.equals( "enumeration" ) )
 			{
 				if ( ! r.hasAttribute( "value" ) )
 				{
 					throw new XMLSchemaParserException
-					( r.getLocalName()
+					( localName
 					  + " in namespace "
 					  + r.getNamespaceURI()
 					  + " does not have attribute"
@@ -283,7 +341,7 @@ public class XMLSchemaTypeParser
 						.addEnumeration( value );
 				}
 			}
-			else if ( r.getLocalName().equals( "annotation" ) )
+			else if ( localName.equals( "annotation" ) )
 			{
 				XMLSchemaTypeParser.checkAnnotation( r );
 			}
