@@ -15,6 +15,7 @@ import daruma.xml.util.ElementUtil;
 import daruma.xml.util.XMLParseErrorException;
 
 import daruma.storage_manager.StorageManager;
+import daruma.storage_manager.StorageAndAuth;
 import daruma.storage_manager.ElementInfo;
 import daruma.storage_manager.StorageException;
 import daruma.storage_manager.type_definition.ElementName;
@@ -22,6 +23,7 @@ import daruma.storage_manager.type_definition.TypeName;
 import daruma.storage_manager.type_definition.TypeDefinition;
 import daruma.storage_manager.type_definition.TypedInstance;
 import daruma.storage_manager.type_definition.TypeException;
+import daruma.storage_manager.type_definition.InstanceParseContext;
 
 import daruma.geometry.TransformationContext;
 
@@ -51,7 +53,9 @@ public class TransactionUpdateHandler
 {
 	private	enum UpdateMode { PRESERVE , OVERRIDE , RESTORE };
 
+	private	StorageAndAuth		storageAndAuth;
 	private	StorageManager		storage;
+	private	InstanceParseContext	parseContext;
 	private	SAXExceptionObserver	errorReceiver;
 
 	private TransactionResultInfo	resultInfo;
@@ -70,13 +74,16 @@ public class TransactionUpdateHandler
 	public	TransactionUpdateHandler( OutputStream  out ,
 					  XMLReader  parser ,
 					  boolean  isTopLevelHandler ,
-					  StorageManager  storage ,
+					  StorageAndAuth  storage ,
+					  InstanceParseContext  parseContext,
 					  SAXExceptionObserver  errorReceiver,
-					  TransactionResultInfo resultInfo)
+					  TransactionResultInfo  resultInfo)
 	{
 		super( out , parser , isTopLevelHandler );
 
-		this.storage              = storage;
+		this.storageAndAuth       = storage;
+		this.storage              = this.storageAndAuth.getStorage();
+		this.parseContext         = parseContext;
 		this.errorReceiver        = errorReceiver;
 		this.specifiedElement     = null;
 		this.specifiedElementType = null;
@@ -319,6 +326,7 @@ public class TransactionUpdateHandler
 						    false ,
 						    this,
 						    this.storage,
+						    this.parseContext,
 						    this.specifiedElementInfo,
 						    this.specifiedElement );
 
@@ -352,7 +360,7 @@ public class TransactionUpdateHandler
 						    this.specifiedElement ,
 						    this.specifiedElementType ,
 						    new TransformationContext() ,
-						    this.storage ,
+						    this.storageAndAuth ,
 						    this );
 
 				super.setContentHandlerDelegator
@@ -489,6 +497,8 @@ public class TransactionUpdateHandler
 		private	SAXExceptionObserver	errorReceiver;
 
 		private	StorageManager		storage;
+		private	InstanceParseContext	parseContext;
+
 		private	ElementInfo		elementInfo;
 		private	ElementName		elementName;
 
@@ -506,6 +516,7 @@ public class TransactionUpdateHandler
 					 boolean  isTopLevelHandler ,
 					 SAXExceptionObserver  errorReceiver,
 					 StorageManager storage,
+					 InstanceParseContext parseContext,
 					 ElementInfo elementInfo,
 					 ElementName elementName )
 		{
@@ -513,6 +524,7 @@ public class TransactionUpdateHandler
 
 			this.errorReceiver   = errorReceiver;
 			this.storage         = storage;
+			this.parseContext    = parseContext;
 			this.elementInfo     = elementInfo;
 			this.elementName     = elementName;
 			this.propertyBuffer  = null;
@@ -595,6 +607,7 @@ public class TransactionUpdateHandler
 						    false,
 						    this,
 						    this.storage,
+						    this.parseContext,
 						    this.elementInfo,
 						    this.elementName,
 						    this.property );
@@ -692,6 +705,7 @@ public class TransactionUpdateHandler
 	    private SAXExceptionObserver errorReceiver;
 
 	    private StorageManager storage;
+	    private InstanceParseContext parseContext;
 	    private ElementInfo elementInfo;
 	    private ElementName topLevelElementName;
 	    private SimpleXPath path;
@@ -703,6 +717,7 @@ public class TransactionUpdateHandler
 				 boolean isTopLevelHandler,
 				 SAXExceptionObserver errorReceiver,
 				 StorageManager storage,
+				 InstanceParseContext parseContext,
 				 ElementInfo elementInfo,
 				 ElementName topLevelElementName,
 				 SimpleXPath path )
@@ -711,6 +726,7 @@ public class TransactionUpdateHandler
 
 		this.errorReceiver       = errorReceiver;
 		this.storage             = storage;
+		this.parseContext        = parseContext;
 		this.elementInfo         = elementInfo;
 		this.topLevelElementName = topLevelElementName;
 		this.path                = path;
@@ -771,7 +787,8 @@ public class TransactionUpdateHandler
 						 this.topLevelElementName,
 						 elementPath,
 						 this.storage,
-						 0 );
+						 0,
+						 this.parseContext );
 		    this.obj = r.getFirst();
 
 		    if ( ElementUtil.hasExtraChildElements
